# coding: utf8

"""{{ pipeline.title }} - Clinica Utilities.
This file has been generated automatically by the `clinica generate template`
command line tool. See here for more details:
http://clinica.run/doc/InteractingWithClinica/
"""


def step1(in_hello_word):
    """Example function for Step 1.
    """

    print(in_hello_word + " from the step 1 of the {{ pipeline.title }} Clinica pipeline.")


def step2(in_hello_word):
    """Example function for Step 2.
    """

    print(in_hello_word + " from the step 2 of the {{ pipeline.title }} Clinica pipeline.")

